/******************************************************************************
*   @file   ad5162.c
*   @brief  Implementation of AD5162 Driver for NIOS processor.
*   @author ACostina (adrian.costina@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 732
******************************************************************************/
#define AD5162_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include <altera_avalon_pio_regs.h>         // PIO access macros.
#include <altera_avalon_spi.h>              // SPI.
#include <unistd.h>                         // usleep() function.
#include <system.h>                         // System address map.
#include <stdio.h>
#include "ad5162.h"
#if (USE_AD5162 == 1)

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char   txBuffer[2]     = {0,};     // Tx SPI Buffer.

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
unsigned char*           rgRdacVal0		    = &AD5162_Data[0].uchData;  //channel 1 potentiometer
unsigned char*           rgRdacVal1      	= &AD5162_Data[1].uchData;  //channel 2 potentiometer
// voltage on W1_Buf if: A20 is AC+DC, A21 is GND, A24 is GND and A25 is 3.3V
float*			 		 voltage1			= &AD5162_Data[2].fData;


/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for AD5162 Board.
 *
 * @param None.
 *
 * @return success
******************************************************************************/
int AD5162_Init(void)
{
    SetCtrl(AD5162_SPI_CFG);                    // enable output buffers of SPORT SPI interface
    IOWR_ALTERA_AVALON_PIO_DATA(GPIO_BASE, 0);  // reset data
    IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, GPIO_RESET
            | GPIO_WP
            | GPIO_MUX_A0
            | GPIO_MUX_A1
            | GPIO_MUX_A2 ); // set output
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_RESET);     // set RESET to HIGH (active LOW)
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_WP);        // set WP to HIGH (active LOW)
    // select the AD5162 SPI line (SYNC-7) as output of SYNC lines MUX
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_MUX_A0|GPIO_MUX_A1 );

    return(SUCCESS);
}

/******************************************************************************
 * @brief Send a command to the device.
 *
 * @param address - will be used to set the channel to be programmed (valid values 0-1)
 * @param inData - will be used to set the value to be set on the channel (valid values 0-255)
 *
 * @return None
******************************************************************************/

void AD5162_SendCommand(unsigned char address, unsigned char inData)
{
    txBuffer[0] = address;
    txBuffer[1] = inData;
    //send command to the device
    alt_avalon_spi_command(AD5162_SPI,  // base address of SPI module
                           0,           // slave number
                           2,           // write_length
                           txBuffer,    // *write_data
                           0,           // read_length
                           NULL,	    // *read_data
                           0);          // flags
    //if the output buffer is not NULL read data from the device
}

/******************************************************************************
 * @brief Update the device's monitored and controlled parameters
 * In case the value to be set on is updated, the new value will be set on the
 * correct channel.
 * It also computes the W1_BUF voltage for the case when the evaluation board
 * jumpers are set as follows: A20 is AC+DC, A21 is GND, A24 is GND and A25
 * is 3.3V
 *
 * @param None
 *
 * @return Always returns success
******************************************************************************/
int AD5162_UpdateParameters(void)
{
    static unsigned char rdac0_last = 0;
    static unsigned char rdac1_last = 0;

    if (rdac0_last != *rgRdacVal0)
    {
        AD5162_SendCommand(0, *rgRdacVal0 );
        rdac0_last = *rgRdacVal0;
    }
    if (rdac1_last != *rgRdacVal1)
    {
        AD5162_SendCommand(1, *rgRdacVal1 );
        rdac1_last = *rgRdacVal1;
    }
    *voltage1 = (rdac0_last * 33.0) / 5120.0;	// ignoring wiper contact resistance Rw
    usleep(1000);								// adding sleep in order to reduce communication with uC-Probe

    return(SUCCESS);
}
#endif /* USE_AD5162 */
